#!/usr/local/bin/perl

use strict;
use warnings;
use IO::File;

print "Please input the allocation table:\n";

my $out = new IO::File "tmpalloc.mp", O_WRONLY|O_CREAT
   or die "unable to open output file: $!\n";
select($out);
copy("malloc.head");

my $alloc_name = "Aa";
my $free_name = "Fa";
my $index = 0;
my $last_free;
my @element;
while (<>) {
   chomp;
   next unless my ($status, $size) = m{^\s*\*?([AF])\s+\d+\s+(\d+)};
   my $name;
   if ($index == 0) {
      $name = "Root";
   } else {
      if ($status eq "F") {
	 $name = $free_name++;
      } else {
	 $name = $alloc_name++;
      }
   }
   push(@element, {
      name => $name,
      status => $status,
      size => $size,
      prev => $index > 0? $element[$index-1]: undef,
   });
   $last_free = $element[$index] if $status eq "F";
   ++$index;
}
die "no useful input found\n" unless $index > 1;
$element[--$index]->{name} = "Last";
my $free;
while ($index >= 0) {
   if ($element[$index]->{status} eq "F") {
      $element[$index]->{target} = $free if defined $free;
      $free = $element[$index];
   }
   --$index;
}

foreach my $element (@element) {
   if ($element->{name} eq "Root") {
      print "RootSegment(Root, ", $element->{target}->{name}, ");\n";
   } elsif ($element->{status} eq "F") {
      if ($last_free eq $element) {
	 print "LastFreeSegment(", $element->{name}, ", ",
	    $element->{prev}->{name},
	    ", btex ", $element->{size}, " etex);\n";
      } else {
	 print "FreeSegment(", $element->{name}, ", ",
	    $element->{prev}->{name}, ", ",
	    $element->{target}->{name},
	    ", btex ", $element->{size}, " etex);\n";
      }
   } else {
      print "AllocatedSegment(", $element->{name}, ", ",
	 $element->{prev}->{name}, ", btex ", $element->{size}, " etex);\n";
   }
}
copy("malloc.tail");
$out->close;
unlink "tmpalloc-1.pdf" if -f "tmpalloc-1.pdf";
system "make tmpalloc.pdf && xpdf tmpalloc.pdf";

sub copy {
   my ($filename) = @_;
   my $in = new IO::File $filename or die "cannot open $filename: $!\n";
   print $_ while <$in>;
   $in->close;
}


__END__

 F 135528     0 --> 135536
*F 135536 16280 --> 151864
 A 151824    32 --> 135536 (f 151832)
 F 151864    16 --> 135528
 A 151888    16 --> 151864 (f 151896)

RootSegment(Root, Fa);
FreeSegment(Fa, Root, Fb, btex 16280 etex);
AllocatedSegment(Aa, Fa, btex 32 etex);
LastFreeSegment(Fb, Aa, btex 16 etex);
AllocatedSegment(Last, Fb, btex 16 etex);
